import java.sql.*;

public class CallTest {
	public static void main(String args[]) throws Exception {
		Class.forName("org.postgresql.Driver");
		setupDB();

		int numClients = 10;
		int numRuns = 1000;
		try {
			numClients = Integer.parseInt(args[0]);
		} catch (Exception e) { }
		try {
			numRuns = Integer.parseInt(args[1]);
		} catch (Exception e) { }

		CallClient clients[] = new CallClient[numClients];
		for (int i=0; i<clients.length; i++) {
			clients[i] = new CallClient(getConnection(), i, numRuns);
			clients[i].open();
		}

		for (int i=0; i<clients.length; i++) {
			clients[i].start();
		}

	}

	private static Connection getConnection() throws SQLException {
		String url = "jdbc:postgresql://localhost:5750/jurka";
		String user = "jurka";
		String passwd = null;
		Connection conn = DriverManager.getConnection(url,user,passwd);
		conn.setAutoCommit(false);
		return conn;
	}

	private static void setupDB() throws SQLException {
		Connection conn = getConnection();
		Statement stmt = conn.createStatement();
		try {
			stmt.executeUpdate("DROP TABLE mytable");
		} catch (SQLException sqle) {
			conn.rollback();
		}
		stmt.executeUpdate("CREATE TABLE mytable(a float)");
		stmt.executeUpdate("CREATE OR REPLACE FUNCTION doquery(int) returns float AS 'SELECT $1*random();' language 'SQL'");
		stmt.executeUpdate("CREATE OR REPLACE FUNCTION doinsert(float) returns int AS 'BEGIN INSERT INTO mytable (a) VALUES ($1); RETURN 1; END;' language 'plpgsql'");
		conn.commit();
		conn.close();
	}

}

class CallClient extends Thread {

	private int clientNumber;
	private int numRuns;
	private Connection conn;
	private CallableStatement queryStatement;
	private CallableStatement insertStatement;

	public CallClient(Connection conn, int clientNumber, int numRuns) {
		this.conn = conn;
		this.clientNumber = clientNumber;
		this.numRuns = numRuns;
	}

	public void open() throws SQLException {
		queryStatement = conn.prepareCall("{ ? = call doQuery(?) }");
		queryStatement.registerOutParameter(1, Types.DOUBLE);
		insertStatement = conn.prepareCall("{ ? = call doInsert(?) }");
		insertStatement.registerOutParameter(1, Types.INTEGER);
	}

	public void close() throws SQLException {
		queryStatement.close();
		insertStatement.close();
		conn.close();
	}

	public void execute(int num) throws SQLException {
		for (int i=0; i<num; i++) {
			queryStatement.setInt(2, i+1);
			queryStatement.execute();
			insertStatement.setDouble(2, queryStatement.getDouble(1));
			insertStatement.execute();
			int result = insertStatement.getInt(1);
		}
		conn.commit();
	}

	public void run() {
		try {
			long start = System.currentTimeMillis();
			execute(numRuns);
			long stop = System.currentTimeMillis();
			close();
			System.out.println("Client: " + clientNumber + " = " + (stop-start));
		} catch (SQLException sqle) {
			sqle.printStackTrace();
		}
	}

}
